/**
 * Titel:     Quelltext zu Kapitel 14 <br />
 * Buch:      Grundkurs-Java fr Wirtschaftsinformatiker. Vieweg 2007. <br />
 * Autoren:   Klaus-Georg Deck und Herbert Neuendorf
 * Copyright: Copyright (c) 2007 <br />
 * 
 * @author Herbert Neuendorf<br />
 * @version 1.0 <br />
 * 
 */
abstract class Konto {	   // abstrakte Oberklasse
      private String name;    
	  private int kontoNr;  
      private double saldo;
      public Konto( String n, int kn, double sd ) {
         name = n;    
		 kontoNr = kn;    
		 saldo = sd;
      }
      public void einzahlen( double betrag ) { 
         saldo = saldo + Math.abs(betrag);
      }
      public String getName() { return name; }
      public int getKontoNr() { return kontoNr; }
      public double getSaldo() { return saldo; }
      public void setSaldo( double betrag ) { saldo = betrag; }
	  
      abstract public boolean abheben( double betrag);
      abstract public void zinsen();
      abstract public void datenAusgabe();
}

class Giro extends Konto {
      private double dispo;      
	  private double sollZins;
      private boolean zuHoch;    
	  private boolean ueberzogen;
      private static int zaehlerGiro = 0;
      public Giro( String n, double sd, double disp, double sZ ) {
         super( n, zaehlerGiro+100, sd );
         zaehlerGiro++;
         dispo = -Math.abs(disp);    
		 sollZins = sZ;
      }
      public boolean abheben( double betrag ) {
         double test = getSaldo() - Math.abs(betrag);
         if( test < dispo ) return false;
         else { setSaldo( test );    return true; }
      }
      public void zinsen() {
         if( getSaldo() < 0.0 ) {    // nur Sollzinsen 
            setSaldo( getSaldo()*(1 + sollZins) );
         }
      }
      public String toString() {
         check();
         String daten = "Girokontodaten:\n";
         daten += "Girokontonummer = " + getKontoNr() + "\n";
         daten += "Inhaber = " + getName() + "\n";
         daten += "Dispo = " + dispo + "\n";
         daten += "Kontostand = " + IO.round( getSaldo(),2 ) + "\n";
         daten += "Sollzinssatz = " + sollZins + "\n";
         daten += "berzogen = " + ueberzogen + "\n";
         daten += "Hochstand = " + zuHoch + "\n";
         return daten;
      }
      public void datenAusgabe(){
    	  IO.writeln( "Daten \n" + this.toString() );
      }
      private void check() {
         ueberzogen = ( getSaldo() < 0.0 );
         zuHoch = ( getSaldo() > 1000.0 ); 
      }
}
   
class Sparkonto extends Konto {
      private double zinsSatz;    
      private static int zaehlerSpar = 0;
      Sparkonto(String n, double sd, double zs) {
         super( n, zaehlerSpar+200, sd );
         zaehlerSpar++;    zinsSatz = zs;
      }
      public boolean abheben( double betrag ) {
         double test = getSaldo() - Math.abs(betrag);
         if (test < 0.0 ) return false;
         else { setSaldo( test );     return true; }
      }
      public void zinsen() { 
	     setSaldo( getSaldo()*(1+zinsSatz) ); 
	  }
      public String toString() {
         String daten = "Sparkontodaten:\n";
         daten += "Sparkontonummer = " + getKontoNr() + "\n";
         daten += "Inhaber = " + getName() + "\n";
         daten += "Kontostand = " + IO.round( getSaldo(), 2) + "\n";
         daten += "Zinsatz = " + zinsSatz + "\n";
         return daten;
      }
      public void datenAusgabe(){
    	  IO.writeln( "Daten \n" + this.toString() );
      }
}
   
class Festzinskonto extends Sparkonto {
      private int laufzeit;     
	  private int abgelaufeneZeit;
      Festzinskonto( String n, double sd, double zs, int lz ) {
         super( n, sd, zs );
         laufzeit = Math.abs(lz);     
		 abgelaufeneZeit = 0;
      }
      public boolean abheben( double betrag ) {
         if( laufzeit > abgelaufeneZeit ) return false;
         return super.abheben( betrag );
      }
      public String toString() {
         String daten = super.toString();
         daten += "Gesamtlaufzeit = " + laufzeit + "\n";
         daten += "Anlagedauer = " + abgelaufeneZeit + "\n";
         return daten;
      }
      public void datenAusgabe(){
    	  IO.writeln( "Daten \n" + this.toString() );
      }
      public void zinsen() { 
	  	 super.zinsen();   
		 abgelaufeneZeit++; }
} 

public class Bank3 {
	public static Konto kontoAnlegen() {
		double limit, zins, dispo, saldo;      
		String name;      
		int laufzeit;
		Konto k;
		IO.writeln( "Funktionsauswahl:" );
		IO.writeln( "=======================" );
		IO.writeln( "Girokonto anlegen           = 1" );
		IO.writeln( "Sparkonto anlegen           = 2" );
		IO.writeln( "Festzinskonto anlegen       = 3" );
		IO.writeln( "Abbrechen durch beliebige andere Zahl" );
		int wahl = IO.promptAndReadInt( "Gewnschte Option = " );
		switch( wahl ) {
			case 1:
				name = IO.promptAndReadString( "Kontoinhaber = " );
				limit = IO.promptAndReadDouble( "Kreditlimit = " );
				saldo = IO.promptAndReadDouble( "Stand = " );
				zins = IO.promptAndReadDouble( "Sollzins = " );
				k = new Giro( name, saldo, limit, zins );
				break;
			case 2:
				name = IO.promptAndReadString( "Kontoinhaber = " );
				zins = IO.promptAndReadDouble( "Zins = " );
				saldo = IO.promptAndReadDouble( "Stand = " );
				k = new Sparkonto( name, saldo, zins );
				break;
			case 3:
				name = IO.promptAndReadString( "Kontoinhaber = " );
				zins = IO.promptAndReadDouble( "Zins = " );
				saldo = IO.promptAndReadDouble( "Stand = " );
				laufzeit = IO.promptAndReadInt( "Laufzeit = " );		
				k = new Festzinskonto( name, saldo, zins, laufzeit );
				break;	
			default: 
				k = null;
				IO.writeln( "Kontoanlage abgebrochen!" );	
		}
		return k;
	}
	
	public static void manipulation( Konto k ) {
		if( k == null ) {
			IO.writeln( "Kein gltiges Konto!" );
			return;
		}
		IO.writeln( "Funktionsauswahl:" );
		IO.writeln( "=======================" );
		IO.writeln( "Einzahlen                = 1" );
		IO.writeln( "Abheben                  = 2" );
		IO.writeln( "Zinsen zuschlagen        = 3" );
		IO.writeln( "Daten ausgeben           = 4" );
		int wahl = IO.promptAndReadInt( "Gewnschte Option = " );
		switch( wahl ) {
			case 1:
				k.einzahlen( IO.promptAndReadDouble( "Betrag = ") );
				break;
			case 2:
				boolean test = k.abheben( IO.promptAndReadDouble( "Auszahlung = " ) ); 
				if( test ) IO.writeln( "Auszahlung durchgefhrt" ); 
				else       IO.writeln( "Keine Auszahlung mglich!" );
				break;
			case 3:
				k.zinsen();
				break;	
			case 4:
				IO.writeln( "Daten\n" + k);
				break;			
		}
	}
	
	public static void main( String[] args) {
		Konto[] konten = new Konto[300];
		do {
			IO.writeln( "Funktionsauswahl:" );
			IO.writeln( "=======================" );
			IO.writeln( "Konto anlegen        = 1" );
			IO.writeln( "Kontobewegung        = 2" );
			IO.writeln( "Alle Konten ausgeben = 3" );
			int wahl = IO.promptAndReadInt( "Gewnschte Option = " );
			switch( wahl ) {
				case 1:
					Konto k = kontoAnlegen();
					if( k != null )  konten[k.getKontoNr()] = k; 
				 	break;
				case 2:
					int n = IO.promptAndReadInt( "Konto Nummer: " );
					if( n>0 && n<konten.length ) 
         				manipulation( konten[n] );
					else    IO.writeln( "Ungltige Kontonummer" );
					break;
				case 3:
					IO.writeln( "Datenausgabe aller Konten:" );
					IO.writeln( "==========================" );
					for( int i=0; i<konten.length; i++ )
						if( konten[i] != null ) konten[i].datenAusgabe();
			}
		} while( IO.promptAndReadChar( "Weiter (j)" ) == 'j' );
		
		IO.writeln( "Bank wird geschlossen" );
	}
}
   





 



   
 








   
 









   


 
 





